#ifndef BOOST_TYPE_TRAITS_CHARACTER_OF_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_CHARACTER_OF_HPP_INCLUDED

#include <boost/type_traits/remove_cv_ref.hpp>
#include <boost/type_traits/remove_extent.hpp>
#include <boost/type_traits/remove_pointer.hpp>
#include <boost/type_traits/is_string.hpp>

namespace boost
{
namespace detail
{
template <typename T, bool Enable = boost::is_string<T>::value || boost::is_character<T>::value>
struct character_of_impl{};

#if !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
template <typename... Ts> struct character_of_impl<std::basic_string_view<Ts...>, true>
{
	using type = typename std::basic_string_view<Ts...>::value_type;
};
#endif

template <typename... Ts> struct character_of_impl<boost::basic_string_view<Ts...>, true>
{
	using type = typename boost::basic_string_view<Ts...>::value_type;
};

template <typename... Ts> struct character_of_impl<boost::basic_string_ref<Ts...>, true>
{
	using type = typename boost::basic_string_ref<Ts...>::value_type;
};

template <std::size_t Size, typename... Ts> struct character_of_impl<boost::static_strings::basic_static_string<Size, Ts...>, true>
{
	using type = typename boost::static_strings::basic_static_string<Size, Ts...>::value_type;
};

template <typename... Ts> struct character_of_impl<std::basic_string<Ts...>, true>
{
	using type = typename std::basic_string<Ts...>::value_type;
};

template <typename T, std::size_t N> struct character_of_impl<T[N], true>
{
	using type = T;
};

template <typename T> struct character_of_impl<T[], true>
{
	using type = T;
};

template <typename T> struct character_of_impl<T* const, true>
{
	using type = T;
};

template <typename T> struct character_of_impl<T*, true>
{
	using type = T;
};

template <typename T> struct character_of_impl<T, true>
{
	using type = T;
};

}	//	detail

template <typename T> struct character_of : public detail::character_of_impl<T>{};

#if !defined(BOOST_NO_CXX11_TEMPLATE_ALIASES)
template <typename T> using character_of_t = typename boost::character_of<T>::type;
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_CHARACTER_OF_HPP_INCLUDED
