#ifndef BOOST_TYPE_TRAITS_IS_BINARY_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_BINARY_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <limits>

namespace boost
{
template <typename T> struct is_binary : public boost::integral_constant
<
	bool,
	std::numeric_limits<T>::radix == 2
>	{};
template <typename T> struct is_binary<const T>			: public boost::is_binary<T>{};
template <typename T> struct is_binary<volatile const T>: public boost::is_binary<T>{};
template <typename T> struct is_binary<volatile T>		: public boost::is_binary<T>{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_binary_v = boost::is_binary<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept binary = boost::is_binary<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_BINARY_HPP_INCLUDED
