#ifndef BOOST_TYPE_TRAITS_IS_BOUNDED_STRING_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_BOUNDED_STRING_HPP_INCLUDED

#include <boost/type_traits/remove_extent.hpp>
#include <boost/type_traits/is_character.hpp>
#include <boost/type_traits/rank.hpp>

namespace boost
{
template <typename T> struct is_bounded_string : public boost::integral_constant
<
	bool,
	boost::is_character<typename boost::remove_cv<typename boost::remove_extent<T>::type>::type>::value &&
	boost::rank<T>::value == 1
>	{};
template <typename T> struct is_bounded_string<const T>			: public boost::is_bounded_string<T>{};
template <typename T> struct is_bounded_string<volatile const T>: public boost::is_bounded_string<T>{};
template <typename T> struct is_bounded_string<volatile T>		: public boost::is_bounded_string<T>{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_bounded_string_v = boost::is_bounded_string<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept bounded_string = boost::is_bounded_string<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_BOUNDED_STRING_HPP_INCLUDED
