#ifndef BOOST_TYPE_TRAITS_IS_CHARACTER_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_CHARACTER_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/type_traits/is_same.hpp>

namespace boost
{
template <typename T> struct is_character					: public boost::false_type{};
template <typename T> struct is_character<const T>			: public boost::is_character<T>{};
template <typename T> struct is_character<volatile const T>	: public boost::is_character<T>{};
template <typename T> struct is_character<volatile T>		: public boost::is_character<T>{};

#if !defined(BOOST_NO_CXX11_CHAR32_T)
template<> struct is_character<char32_t>	: public boost::true_type{};
#endif

#if !defined(BOOST_NO_CXX11_CHAR16_T)
template<> struct is_character<char16_t>	: public boost::true_type{};
#endif

#if defined(__cpp_char8_t)
template<> struct is_character<char8_t>		: public boost::true_type{};
#endif

template<> struct is_character<wchar_t>		: public boost::true_type{};
template<> struct is_character<char>		: public boost::true_type{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_character_v = boost::is_character<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept character = boost::is_character<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_CHARACTER_HPP_INCLUDED
