#ifndef BOOST_TYPE_TRAITS_IS_INTEGER_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_INTEGER_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/cstdint.hpp>

namespace boost
{
template <typename T> struct is_integer						: public boost::false_type{};
template <typename T> struct is_integer<const T>			: public boost::is_integer<T>{};
template <typename T> struct is_integer<volatile const T>	: public boost::is_integer<T>{};
template <typename T> struct is_integer<volatile T>			: public boost::is_integer<T>{};

#if defined(BOOST_HAS_INT128)
template<> struct is_integer<boost::uint128_type>	: public boost::true_type{};
template<> struct is_integer<boost::int128_type>	: public boost::true_type{};
#endif

#if !defined(BOOST_NO_INT64_T)
template<> struct is_integer<boost::uint64_t>	: public boost::true_type{};
template<> struct is_integer<boost::int64_t>	: public boost::true_type{};
#endif

template<> struct is_integer<boost::uint32_t>	: public boost::true_type{};
template<> struct is_integer<boost::int32_t>	: public boost::true_type{};

template<> struct is_integer<boost::uint16_t>	: public boost::true_type{};
template<> struct is_integer<boost::int16_t>	: public boost::true_type{};

template<> struct is_integer<boost::uint8_t>	: public boost::true_type{};
template<> struct is_integer<boost::int8_t>		: public boost::true_type{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_integer_v = boost::is_integer<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept integer = boost::is_integer<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_INTEGER_HPP_INCLUDED
