#ifndef BOOST_TYPE_TRAITS_IS_NATIVE_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_NATIVE_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/cstdfloat.hpp>
#include <boost/cstdint.hpp>

namespace boost
{
template <typename T> struct is_native						: public boost::false_type{};
template <typename T> struct is_native<const T>				: public boost::is_native<T>{};
template <typename T> struct is_native<volatile const T>	: public boost::is_native<T>{};
template <typename T> struct is_native<volatile T>			: public boost::is_native<T>{};

//	character
#if !defined(BOOST_NO_CXX11_CHAR32_T) && (__WORDSIZE >= 32)
template<> struct is_native<char32_t>			: public boost::true_type{};
#endif

#if !defined(BOOST_NO_CXX11_CHAR16_T) && (__WORDSIZE >= 16)
template<> struct is_native<char16_t>			: public boost::true_type{};
#endif

#if defined(__cpp_char8_t) && (__WORDSIZE >= 8)
template<> struct is_native<char8_t>			: public boost::true_type{};
#endif

//	https://en.cppreference.com/w/cpp/language/types
#if defined(BOOST_OS_WINDOWS)
	#if (__WORDSIZE >= 16)
		template<> struct is_native<wchar_t>	: public boost::true_type{};
	#endif
#else
	#if (__WORDSIZE >= 32)
		template<> struct is_native<wchar_t>	: public boost::true_type{};
	#endif
#endif

#if (__WORDSIZE >= 8)
template<> struct is_native<char>				: public boost::true_type{};
#endif



//	integer
#if defined(BOOST_HAS_INT128) && (__WORDSIZE >= 128)
template<> struct is_native<boost::uint128_type>: public boost::true_type{};
template<> struct is_native<boost::int128_type>	: public boost::true_type{};
#endif

#if !defined(BOOST_NO_INT64_T) && (__WORDSIZE >= 64)
template<> struct is_native<boost::uint64_t>: public boost::true_type{};
template<> struct is_native<boost::int64_t>	: public boost::true_type{};
#endif

#if (__WORDSIZE >= 32)
template<> struct is_native<boost::uint32_t>: public boost::true_type{};
template<> struct is_native<boost::int32_t>	: public boost::true_type{};
#endif

#if (__WORDSIZE >= 16)
template<> struct is_native<boost::uint16_t>: public boost::true_type{};
template<> struct is_native<boost::int16_t>	: public boost::true_type{};
#endif

#if (__WORDSIZE >= 8) && (__CHAR_BIT__ == 8)
template<> struct is_native<boost::uint8_t>	: public boost::true_type{};
template<> struct is_native<boost::int8_t>	: public boost::true_type{};
#endif



//	float
//	https://en.wikipedia.org/wiki/SPARC#History
#if defined(BOOST_HAS_FLOAT128) && defined(BOOST_ARCH_SPARC)
	#if (BOOST_ARCH_SPARC >= 8)
		template<> struct is_native<boost::float128_t>	: public boost::true_type{};
	#endif
#endif

#if defined(BOOST_ARCH_SPARC)
	#if (BOOST_ARCH_SPARC < 8)
		template<> struct is_native<boost::float80_t>	: public boost::true_type{};
	#endif
#endif

#if defined(BOOST_ARCH_X86)
template<> struct is_native<boost::float80_t>			: public boost::true_type{};
#endif

template<> struct is_native<boost::float64_t>			: public boost::true_type{};
template<> struct is_native<boost::float32_t>			: public boost::true_type{};

#if defined(BOOST_HAS_FLOAT16)
	#if defined(BOOST_ARCH_ARM)
		template<> struct is_native<boost::float16_t>	: public boost::true_type{};
	#endif
#endif

#if defined(BOOST_HAS_FLOAT16B)
	#if defined(???)
		template<> struct is_native<boost::float16b_t>	: public boost::true_type{};
	#endif
#endif


#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_native_v = boost::is_native<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept native = boost::is_native<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_NATIVE_HPP_INCLUDED
