#ifndef BOOST_TYPE_TRAITS_IS_NUMERIC_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_NUMERIC_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/type_traits/is_floating_point.hpp>
#include <boost/type_traits/is_complex.hpp>
#include <boost/type_traits/is_integer.hpp>
#include <boost/type_traits/is_rational.hpp>

namespace boost
{
template <typename T> struct is_numeric : public boost::integral_constant
<
	bool,
	boost::is_floating_point<T>::value	||
	boost::is_integer<T>::value			||
	boost::is_complex<T>::value			||
	boost::is_rational<T>::value
>	{};
template <typename T> struct is_numeric<const T>			: public boost::is_numeric<T>{};
template <typename T> struct is_numeric<volatile const T>	: public boost::is_numeric<T>{};
template <typename T> struct is_numeric<volatile T>			: public boost::is_numeric<T>{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_numeric_v = boost::is_numeric<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept numeric = boost::is_numeric<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_NUMERIC_HPP_INCLUDED
