#ifndef BOOST_TYPE_TRAITS_IS_STRING_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_STRING_HPP_INCLUDED

#include <boost/type_traits/is_unbounded_string.hpp>
#include <boost/type_traits/is_bounded_string.hpp>
#include <boost/type_traits/is_static_string.hpp>
#include <boost/type_traits/is_string_class.hpp>
#include <boost/type_traits/is_string_view.hpp>
#include <boost/type_traits/is_string_ref.hpp>

namespace boost
{
template <typename T> struct is_string : public boost::integral_constant
<
	bool,
	boost::is_unbounded_string<T>::value	||
	boost::is_bounded_string<T>::value		||
	boost::is_static_string<T>::value		||
	boost::is_string_class<T>::value		||
	boost::is_string_view<T>::value			||
	boost::is_string_ref<T>::value
>	{};
template <typename T> struct is_string<const T>			: public boost::is_string<T>{};
template <typename T> struct is_string<volatile const T>: public boost::is_string<T>{};
template <typename T> struct is_string<volatile T>		: public boost::is_string<T>{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_string_v = boost::is_string<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept string = boost::is_string<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_STRING_HPP_INCLUDED
