#ifndef BOOST_TYPE_TRAITS_IS_TUPLE_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_TUPLE_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/tuple/tuple.hpp>
#include <tuple>

namespace boost
{
template <typename T> struct is_tuple 					: public boost::false_type{};
template <typename T> struct is_tuple<const T>			: public boost::is_tuple<T>{};
template <typename T> struct is_tuple<volatile const T>	: public boost::is_tuple<T>{};
template <typename T> struct is_tuple<volatile T>		: public boost::is_tuple<T>{};

template <typename... Args>	struct is_tuple<std::tuple<Args...>>	: public boost::true_type{};
template <typename... Args>	struct is_tuple<boost::tuple<Args...>>	: public boost::true_type{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_tuple_v = boost::is_tuple<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept tuple = boost::is_tuple<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_TUPLE_HPP_INCLUDED
