#ifndef BOOST_TYPE_TRAITS_IS_UNBOUNDED_STRING_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_UNBOUNDED_STRING_HPP_INCLUDED

#include <boost/type_traits/remove_cv_ptr.hpp>
#include <boost/type_traits/is_character.hpp>
#include <boost/type_traits/is_pointer.hpp>

namespace boost
{
template <typename T> struct is_unbounded_string : public boost::integral_constant
<
	bool,
	boost::is_character<typename boost::remove_cv_ptr<T>::type>::value &&
	boost::is_pointer<T>::value
>	{};
template <typename T> struct is_unbounded_string<const T>			: public boost::is_unbounded_string<T>{};
template <typename T> struct is_unbounded_string<volatile const T>	: public boost::is_unbounded_string<T>{};
template <typename T> struct is_unbounded_string<volatile T>		: public boost::is_unbounded_string<T>{};

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_unbounded_string_v = boost::is_unbounded_string<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept unbounded_string = boost::is_unbounded_string<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_UNBOUNDED_STRING_HPP_INCLUDED
