#ifndef BOOST_TYPE_TRAITS_IS_VECTORISABLE_HPP_INCLUDED
#define BOOST_TYPE_TRAITS_IS_VECTORISABLE_HPP_INCLUDED

#include <boost/type_traits/integral_constant.hpp>
#include <boost/cstdfloat.hpp>
#include <boost/cstdint.hpp>

namespace boost
{
template <typename T> struct is_vectorisable					: public boost::false_type{};
template <typename T> struct is_vectorisable<const T>			: public boost::is_vectorisable<T>{};
template <typename T> struct is_vectorisable<volatile const T>	: public boost::is_vectorisable<T>{};
template <typename T> struct is_vectorisable<volatile T>		: public boost::is_vectorisable<T>{};

#if (BOOST_HW_SIMD_X86 >= BOOST_HW_SIMD_X86_SSE_VERSION)
template<> struct is_vectorisable<boost::float64_t>: public boost::true_type{};
template<> struct is_vectorisable<boost::float32_t>: public boost::true_type{};
#endif

#if (BOOST_HW_SIMD_X86 >= BOOST_HW_SIMD_X86_SSE2_VERSION)
template<> struct is_vectorisable<boost::uint64_t>	: public boost::true_type{};
template<> struct is_vectorisable<boost::int64_t>	: public boost::true_type{};

template<> struct is_vectorisable<boost::uint32_t>	: public boost::true_type{};
template<> struct is_vectorisable<boost::int32_t>	: public boost::true_type{};

template<> struct is_vectorisable<boost::uint16_t>	: public boost::true_type{};
template<> struct is_vectorisable<boost::int16_t>	: public boost::true_type{};

template<> struct is_vectorisable<boost::uint8_t>	: public boost::true_type{};
template<> struct is_vectorisable<boost::int8_t>	: public boost::true_type{};

#if !defined(BOOST_NO_CXX11_CHAR32_T)
template<> struct is_vectorisable<char32_t>	: public boost::true_type{};
#endif

#if !defined(BOOST_NO_CXX11_CHAR16_T)
template<> struct is_vectorisable<char16_t>	: public boost::true_type{};
#endif

#if defined(__cpp_char8_t)
template<> struct is_vectorisable<char8_t>	: public boost::true_type{};
#endif

template<> struct is_vectorisable<wchar_t>	: public boost::true_type{};
template<> struct is_vectorisable<char>		: public boost::true_type{};
#endif

/*
#if (BOOST_HW_SIMD_ARM >= ???)
#endif
*/

/*
#if (BOOST_HW_SIMD_PPC >= ???)
#endif
*/

#if !defined(BOOST_NO_CXX17_INLINE_VARIABLES)
template <typename T> BOOST_INLINE_CONSTEXPR bool is_vectorisable_v = boost::is_vectorisable<T>::value;
#endif

#if defined(__cpp_concepts)
namespace meta
{
template <typename T> concept vectorisable = boost::is_vectorisable<T>::value;
}	//	meta
#endif
}	//	boost

#endif	//	BOOST_TYPE_TRAITS_IS_VECTORISABLE_HPP_INCLUDED
